package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.OrganisationBlurringEntity;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;
import java.time.*;
import java.util.List;

import static org.junit.Assert.assertEquals;

public class OrganisationBlurringDaoTest extends AbstractDaoTest {
    @Autowired
    DataSource dataSource;

    private OrganisationBlurringDao organisationBlurringDao;

    private final OrganisationBlurringEntity TEST_BLURRING_1 =
            new OrganisationBlurringEntity("1111", "CVR", "12345678",
                    instantFrom(2022, 1, 1, 0, 0), instantFrom(3000, 1, 1, 0, 0));

    private final OrganisationBlurringEntity TEST_BLURRING_EXPIRED =
            new OrganisationBlurringEntity("5555", "CVR", "11112222",
                    instantFrom(2020, 1, 1, 0, 0), instantFrom(2021, 1, 1, 0, 0));

    @Before
    public void setUp() {
        organisationBlurringDao = new OrganisationBlurringDao(dataSource);
    }

    @Test
    public void getAllOrganisationBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getAllOrganisationBlurrings("12345678");

        // Assert
        assertEquals(1, blurrings.size());
        OrganisationBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_1.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_1.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getAllOrganisationBlurrings_WhenNoMatchingBlurringIsFound_ReturnsEmptyList() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getAllOrganisationBlurrings("does not exist");

        // Assert
        assertEquals(0, blurrings.size());
    }

    @Test
    public void getAllOrganisationBlurrings_WhenOneMatchingExpiredBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getAllOrganisationBlurrings("11112222");

        // Assert
        assertEquals(1, blurrings.size());
        OrganisationBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_EXPIRED.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_EXPIRED.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getActiveOrganisationBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getActiveOrganisationBlurrings("12345678");

        // Assert
        assertEquals(1, blurrings.size());
        OrganisationBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(TEST_BLURRING_1.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(TEST_BLURRING_1.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    @Test
    public void getActiveOrganisationBlurrings_WhenOneMatchingExpiredBlurringIsFound_ReturnsEmptyList() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getActiveOrganisationBlurrings("11112222");

        // Assert
        assertEquals(0, blurrings.size());
    }

    @Test
    public void getActiveOrganisationBlurrings_WhenNoMatchingBlurringIsFound_ReturnsEmptyList() {
        // Act
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getActiveOrganisationBlurrings("does not exist");

        // Assert
        assertEquals(0, blurrings.size());
    }

    @Test
    public void insert_WhenCalled_InsertsBlurring() {
        // Arrange
        OrganisationBlurringEntity blurringEntity = new OrganisationBlurringEntity("2222", "CVR", "12345678",
                instantFrom(2022, 1, 1, 0, 0), instantFrom(3000, 1, 1, 0, 0));

        // Act
        organisationBlurringDao.insert(blurringEntity);

        // Assert
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getAllOrganisationBlurrings("12345678");
        assertEquals(2, blurrings.size());
    }

    @Test
    public void insert_WhenCalled_InsertsBlurringWithCorrectValues() {
        // Arrange
        OrganisationBlurringEntity blurringEntity = new OrganisationBlurringEntity("9876", "CVR", "99887766",
                instantFrom(2022, 1, 1, 0, 0), instantFrom(3001, 2, 2, 0, 0));

        // Act
        organisationBlurringDao.insert(blurringEntity);

        // Assert
        List<OrganisationBlurringEntity> blurrings = organisationBlurringDao.getAllOrganisationBlurrings("99887766");
        assertEquals(1, blurrings.size());
        OrganisationBlurringEntity returnedBlurring = blurrings.get(0);
        assertEquals(blurringEntity.getOrganisationId(), returnedBlurring.getOrganisationId());
        assertEquals(blurringEntity.getOrganisationIdType(), returnedBlurring.getOrganisationIdType());
    }

    private static Instant instantFrom(int year, int month, int dayOfMonth, int hour, int minute) {
        return LocalDateTime.of(year, month, dayOfMonth, hour, minute).atZone(ZoneId.systemDefault()).toInstant();
    }
}
